@@ -17,7 +17,7 @@ class EventHandler():
     def __init__(self):
         self.events = []
 
-    def bindEvent(self, event, callback, targetElement):
+    def bindEvent(self, event: str, callback, targetElement):
         """[binds an event to a callback]
 
         Args:
@@ -170,7 +170,7 @@ class dQuery_el():
         # return self
         raise NotImplementedError
 
-    def addBack(self):
+    def addBack(self, selector):
         """ Add the previous set of elements on the stack to the current set, optionally filtered by a selector."""
         raise NotImplementedError
 
@@ -203,27 +203,27 @@ class dQuery_el():
                     p.args = tuple(l)
         return self
 
-    def ajaxComplete(self):
-        """ Register a handler to be called when Ajax requests complete. This is an AjaxEvent."""
+    def ajaxComplete(self, handler):
+        """ Register a handler to be called when Ajax requests complete. This is an Ajax Event"""
         raise NotImplementedError
 
-    def ajaxError(self):
-        """ Register a handler to be called when Ajax requests complete with an error. This is an Ajax Event."""
+    def ajaxError(self, handler):
+        """ Register a handler to be called when Ajax requests complete with an error. This is an Ajax Event"""
         raise NotImplementedError
 
-    def ajaxSend(self):
-        """ Attach a function to be executed before an Ajax request is sent. This is an Ajax Event."""
+    def ajaxSend(self, handler):
+        """ Register a handler to be called when Ajax requests complete successfully. This is an Ajax Event"""
         raise NotImplementedError
 
-    def ajaxStart(self):
-        """ Register a handler to be called when the first Ajax request begins. This is an Ajax Event."""
+    def ajaxStart(self, handler):
+        """ Register a handler to be called when the first Ajax request begins. This is an Ajax Event"""
         raise NotImplementedError
 
-    def ajaxStop(self):
-        """ Register a handler to be called when all Ajax requests have completed. This is an Ajax Event."""
+    def ajaxStop(self, handler):
+        """ Register a handler to be called when all Ajax requests have completed. This is an Ajax Event"""
         raise NotImplementedError
 
-    def ajaxSuccess(self):
+    def ajaxSuccess(self, handler):
         """ Attach a function to be executed whenever an Ajax request completes successfully. This is an Ajax Event."""
         raise NotImplementedError
 
@@ -333,7 +333,7 @@ class dQuery_el():
         # return self
         raise NotImplementedError
 
-    def click(self):
+    def click(self, handler):
         """ Bind an event handler to the “click” JavaScript event, or trigger that event on an element."""
         raise NotImplementedError
 
@@ -350,7 +350,7 @@ class dQuery_el():
         and traversing up through its ancestors in the DOM tree."""
         raise NotImplementedError
 
-    def contents(self):
+    def contents(self, selector=None):
         """ Get the children of each element in the set of matched elements, including text and comment nodes."""
         raise NotImplementedError
 
@@ -358,31 +358,41 @@ class dQuery_el():
     def context(self):
         """ The DOM node context originally passed to dQuery if none was passed
         then context will likely be the document. """
-        raise NotImplementedError
+        from domonic.dom import document
+        return document
 
     def contextmenu(self):
         """ Bind an event handler to the “contextmenu” JavaScript event, or trigger that event on an element."""
         raise NotImplementedError
 
-    def css(self, prop, value):
+    def css(self, property, value=None):  # TODO - untested
         """ Get the value of a computed style property for the first element in the set of matched elements
         or set one or more CSS properties for every matched element."""
-        pass
+        if not isinstance(self.elements, (list, tuple)):
+            self.elements = (self.elements,)
+        if value is not None:
+            if self.elements[0].style.getProperty(property) is not None:
+                self.elements[0].style.setProperty(property, value)
+                return self
+        if type(self.elements) is not tuple and type(self.elements) is not list:
+            return self.elements.style.getProperty(property)
+        else:
+            return self.elements[0].style.getProperty(property)
 
-    def data(self):
+    def data(self, key, value=None):
         """ Store arbitrary data associated with the matched elements or return the value at the named data store
         for the first element in the set of matched elements."""
         raise NotImplementedError
 
-    def dblclick(self):
+    def dblclick(self, handler):
         """ Bind an event handler to the “dblclick” JavaScript event, or trigger that event on an element."""
         raise NotImplementedError
 
-    def delay(self):
+    def delay(self, time):
         """ Set a timer to delay execution of subsequent items in the queue."""
         raise NotImplementedError
 
-    def delegate(self):
+    def delegate(self, selector, event, handler):
         """ Attach a handler to one or more events for all elements that match the selector, now or in the future,
         based on a specific set of root elements."""
         raise NotImplementedError
@@ -391,9 +401,19 @@ class dQuery_el():
         """ Execute the next function on the queue for the matched elements."""
         raise NotImplementedError
 
-    def detach(self):
+    def detach(self):  # TODO - test
         """ Remove the set of matched elements from the DOM."""
-        raise NotImplementedError
+        if not isinstance(self.elements, (list, tuple)):
+            self.elements = (self.elements,)
+
+        for el in self.elements:
+            p = el.parentNode
+            for i, n in enumerate(p.children):
+                if n == el:
+                    l = list(p.args)
+                    l.pop(i)
+                    p.args = tuple(l)
+        return self
 
     def die(self):
         """ Remove event handlers previously attached using .live from the elements."""
@@ -426,13 +446,15 @@ class dQuery_el():
         """ Reduce the set of matched elements to the one at the specified index."""
         return self.elements[index]
 
-    def error(self):
+    def error(self, handler):
         """ Bind an event handler to the “error” JavaScript event."""
         raise NotImplementedError
 
-    def even(self):
+    def even(self):  # TODO - untested
         """ Reduce the set of matched elements to the even ones in the set, numbered from zero."""
-        raise NotImplementedError
+        if not isinstance(self.elements, (list, tuple)):
+            self.elements = (self.elements,)
+        return [el for el in self.elements if el.index % 2 == 0]
 
     def fadeIn(self):
         """ Display the matched elements by fading them to opaque."""
@@ -450,13 +472,20 @@ class dQuery_el():
         """ Display or hide the matched elements by animating their opacity."""
         raise NotImplementedError
 
-    def filter(self):
+    def filter(self, selector):  # TODO - untested
         """ Reduce the set of matched elements to those that match the selector or pass the function’s test."""
-        raise NotImplementedError
+        if isinstance(selector, str):
+            return self.elements.filter(selector)
+        elif callable(selector):
+            return [el for el in self.elements if selector(el)]
+        else:
+            raise TypeError("selector must be a string or a callable")
 
-    def find(self):
+    def find(self, selector):
         """ Get the descendants of each element in the current set of matched elements, filtered by a selector,
         dQuery object, or element."""
+        # if isinstance(selector, str):
+        # return self.elements.find(selector)
         raise NotImplementedError
 
     def finish(self):
@@ -486,10 +515,15 @@ class dQuery_el():
     #     """ Retrieve the DOM elements matched by the dQuery object."""
     #     raise NotImplementedError
 
-    def has(self):
+    def has(self, selector):  # TODO - test
         """ Reduce the set of matched elements to those that have a descendant
         that matches the selector or DOM element."""
-        raise NotImplementedError
+        if isinstance(selector, str):
+            return self.elements.has(selector)
+        elif callable(selector):
+            return [el for el in self.elements if selector(el)]
+        else:
+            raise TypeError("selector must be a string or a callable")
 
     def hasClass(self, classname):
         """ Determine whether any of the matched elements are assigned the given class."""
@@ -612,9 +646,23 @@ class dQuery_el():
         """ Attach an event handler for all elements which match the current selector, now and in the future."""
         raise NotImplementedError
 
-    def load(self):
+    def load(self, url, data=None, complete=None):  # TODO - test
         """ Load data from the server and place the returned HTML into the matched elements."""
-        raise NotImplementedError
+        if data is None:
+            data = {}
+        if complete is None:
+            complete = lambda x: x
+        for el in self.elements:
+            el.innerHTML = ""
+            el.innerHTML = "<div class='loading'></div>"
+            def load(el, url, data, complete):
+                def onload(response):
+                    el.innerHTML = ""
+                    el.innerHTML = response
+                    complete(response)
+                dQuery.ajax(url, data, onload)
+            load(el, url, data, complete)
+        return self
 
     def map(self, func):  # TODO - test
         """ Pass each element in the current matched set through a function,
@@ -669,6 +717,11 @@ class dQuery_el():
     def nextAll(self, selector):
         """ Get all following siblings of each element in the set of matched elements,
         optionally filtered by a selector."""
+        # if not isinstance(self.elements, (list, tuple)):
+        #     self.elements = (self.elements,)
+        # for el in self.elements:
+        #     el.nextAll(selector)
+        # return self
         raise NotImplementedError
 
     def nextUntil(self, selector):
@@ -680,16 +733,16 @@ class dQuery_el():
         """ Remove elements from the set of matched elements."""
         # raise NotImplementedError
 
-    def odd(self):
+    def odd(self):  # TODO - untested
         """ Reduce the set of matched elements to the odd ones in the set, numbered from zero."""
-        raise NotImplementedError
+        return self.filter(lambda x: x % 2 == 1)
 
     def off(self, event):
         """ Remove an event handler."""
         for el in self.elements:
             self.eventHandler.unbindEvent(event, el)
 
-    def offset(self):
+    def offset(self, coordinates):
         """ Get the current coordinates of the first element, or set the coordinates of every element,
         in the set of matched elements, relative to the document."""
         raise NotImplementedError
@@ -732,14 +785,24 @@ class dQuery_el():
                 parents.append(p)
         return parents
 
-    def parents(self):
+    def parents(self, selector):  # TODO - untested
         """ Get the ancestors of each element in the current set of matched elements,
         optionally filtered by a selector."""
-        raise NotImplementedError
+        if not isinstance(self.elements, (list, tuple)):
+            self.elements = (self.elements,)
+        parents = []
+        if isinstance(selector, str):
+            for el in self.elements:
+                p = el.parents
+                parents.append(p)
+        return parents
 
-    def parentsUntil(self):
+    def parentsUntil(self, selector):
         """ Get the ancestors of each element in the current set of matched elements,
         up to but not including the element matched by the selector, DOM node, or dQuery object."""
+        # if not isinstance(self.elements, (list, tuple)):
+        #     self.elements = (self.elements,)
+        # parents = []
         raise NotImplementedError
 
     def position(self):
@@ -764,19 +827,43 @@ class dQuery_el():
             target.append(el)
         return self
 
-    def prev(self):
+    def prev(self, selector):  # TODO - untested
         """ Get the immediately preceding sibling of each element in the set of matched elements.
         If a selector is provided, it retrieves the previous sibling only if it matches that selector."""
+        # if not isinstance(self.elements, (list, tuple)):
+        #     self.elements = (self.elements,)
+        # prevs = []
+        # for el in self.elements:
+        #     prev = el.prev
+        #     # prevs.append(prev)
+        #     if selector:
+        #         if prev.matches(selector):
+        #             prevs.append(prev)
+        # return prevs
         raise NotImplementedError
 
-    def prevAll(self):
+    def prevAll(self, selector):  # TODO - untested
         """ Get all preceding siblings of each element in the set of matched elements,
         optionally filtered by a selector."""
+        # if not isinstance(self.elements, (list, tuple)):
+        #     self.elements = (self.elements,)
+        # prevs = []
+        # for el in self.elements:
+        #     prev = el.prevAll(selector)
+        #     prevs.append(prev)
+        # return prevs
         raise NotImplementedError
 
-    def prevUntil(self):
+    def prevUntil(self, selector):  # TODO - untested
         """ Get all preceding siblings of each element up to but not including the element matched by the selector,
         DOM node, or dQuery object."""
+        # if not isinstance(self.elements, (list, tuple)):
+        #     self.elements = (self.elements,)
+        # prevs = []
+        # for el in self.elements:
+        #     prev = el.prevUntil(selector)
+        #     prevs.append(prev)
+        # return prevs
         raise NotImplementedError
 
     def promise(self):
@@ -796,7 +883,7 @@ class dQuery_el():
         else:
             return self.elements[0].getAttribute(property)
 
-    def pushStack(self, stack):  # TODO - test
+    def pushStack(self, stack):  # TODO - test
         """ Add a collection of DOM elements onto the dQuery."""
         if not isinstance(self.elements, (list, tuple)):
             self.elements = (self.elements,)
@@ -808,7 +895,7 @@ class dQuery_el():
         """ Show or manipulate the queue of functions to be executed on the matched elements."""
         raise NotImplementedError
 
-    def ready(self):
+    def ready(self, callback):
         """ Specify a function to execute when the DOM is fully loaded."""
         raise NotImplementedError
 
@@ -842,7 +929,7 @@ class dQuery_el():
                     el.setAttribute("class", removed)
         return self
 
-    def removeData(self):
+    def removeData(self, name):
         """ Remove a previously-stored piece of data."""
         raise NotImplementedError
 
@@ -854,11 +941,11 @@ class dQuery_el():
             el.removeAttribute(prop)
         return self
 
-    def replaceAll(self):
+    def replaceAll(self, elements):  # TODO - untested
         """ Replace each target element with the set of matched elements."""
         raise NotImplementedError
 
-    def replaceWith(self, replacement):  # TODO - test
+    def replaceWith(self, replacement):  # TODO - test
         """ Replace each element in the set of matched elements with the provided new content and return the set
          of elements that was removed."""
         if not isinstance(self.elements, (list, tuple)):
@@ -869,11 +956,11 @@ class dQuery_el():
             el.parentNode.replaceChild(replacement, el)
         return self
 
-    def resize(self):
+    def resize(self, callback):
         """ Bind an event handler to the “resize” JavaScript event, or trigger that event on an element."""
         raise NotImplementedError
 
-    def scroll(self):
+    def scroll(self, callback):
         """ Bind an event handler to the “scroll” JavaScript event, or trigger that event on an element."""
         raise NotImplementedError
 
@@ -887,7 +974,7 @@ class dQuery_el():
         or set the vertical position of the scroll bar for every matched element."""
         raise NotImplementedError
 
-    def select(self):
+    def select(self, selector):
         """ Bind an event handler to the “select” JavaScript event, or trigger that event on an element."""
         raise NotImplementedError
 
@@ -1053,10 +1140,15 @@ class dQuery_el():
         """ Bind an event handler to the “unload” JavaScript event."""
         raise NotImplementedError
 
-    def unwrap(self):
+    def unwrap(self):  # TODO - untested
         """ Remove the parents of the set of matched elements from the DOM,
         leaving the matched elements in their place."""
-        raise NotImplementedError
+        if not isinstance(self.elements, (list, tuple)):
+            self.elements = (self.elements,)
+        for el in self.elements:
+            if el.parentNode.parentNode:
+                el.parentNode.parentNode.replaceChild(el, el.parentNode)
+        return self
 
     def val(self, newVal=None):
         """ Get the current value of the first element in the set of matched elements
@@ -1073,11 +1165,19 @@ class dQuery_el():
         or set the width of every matched element."""
         raise NotImplementedError
 
-    def wrap(self):
+    def wrap(self, wrappingElement):  # TODO - untested
         """ Wrap an HTML structure around each element in the set of matched elements."""
-        raise NotImplementedError
+        if isinstance(wrappingElement, str):
+            from domonic.html import create_element
+            wrappingElement = create_element(wrappingElement)
+        if not isinstance(self.elements, (list, tuple)):
+            self.elements = (self.elements,)
+        for el in self.elements:
+            wrappingElement.appendChild(el)
+            el.parentNode.replaceChild(wrappingElement, el)
+        return self
 
-    def wrapAll(self):
+    def wrapAll(self, wrappingElement):
         """ Wrap an HTML structure around all elements in the set of matched elements."""
         raise NotImplementedError
 
@@ -1308,7 +1408,14 @@ class º(dQuery_el):
         return dproxy(args[0])
 
     @staticmethod
-    def ajax(url='/', type='GET', data=None, contentType=False, processData=False, cache=False, success=None, error=None):
+    def ajax(url='/',
+             type='GET',
+             data=None,
+             contentType=False,
+             processData=False,
+             cache=False,
+             success=None,
+             error=None):
         """ make an ajax request """
         try:
             # r = requests.get(url, timeout=3)
@@ -1423,9 +1530,10 @@ class º(dQuery_el):
         raise NotImplementedError
 
     @staticmethod
-    def each(arr, func):
+    def each(arr, func):  # TODO - untested
         """ A generic iterator function, which can be used to seamlessly iterate over both objects and arrays. """
-        raise NotImplementedError
+        for i in arr:
+            func(i)
 
     @staticmethod
     def error(msg):
@@ -1552,7 +1660,8 @@ class º(dQuery_el):
     @staticmethod
     def isXMLDoc(obj):
         """ Check to see if a DOM node is within an XML document (or is an XML document). """
-        raise NotImplementedError
+        obj = str(obj)
+        return obj.startswith('<') and obj.endswith('>')
 
     @staticmethod
     def makeArray(somelist):
@@ -1601,19 +1710,26 @@ class º(dQuery_el):
             raise TypeError(obj)
 
     @staticmethod
-    def parseHTML():
+    def parseHTML(string):
         """ Parses a string into an array of DOM nodes. """
-        raise NotImplementedError
+        # from bs4 import BeautifulSoup
+        # return BeautifulSoup(string, 'html.parser')
+        # return BeautifulSoup(string, 'html5lib')
+        # return BeautifulSoup(string, 'lxml')
+        # return domonic.domonic.parseString(string, 'domonic')
+        # return domonic.domonic.parseString(string, 'expat')
+        return domonic.domonic.parseString(string)
 
     @staticmethod
-    def parseJSON():
+    def parseJSON(string: str):
         """ Takes a well-formed JSON string and returns the resulting JavaScript value. """
         raise NotImplementedError
 
     @staticmethod
-    def parseXML():
-        """ Parses a string into an XML document. """
-        raise NotImplementedError
+    def parseXML(string: str):
+        """ Parses an XMLstring into a pyml """
+        from domonic.domonic import parseString
+        return parseString(string)
 
     @staticmethod
     def post(url, data, success):
@@ -1629,7 +1745,7 @@ class º(dQuery_el):
         raise NotImplementedError
 
     @staticmethod
-    def queue():
+    def queue(func):
         """ Show or manipulate the queue of functions to be executed on the matched element. """
         raise NotImplementedError
 
