@@ -1,11 +1,11 @@
 """
-    test_domonic
+    test_components
     ~~~~~~~~~~~~
-    unit tests for css
+    tests for components
 """
 
 import unittest
-import json
+# import json
 # import requests
 # from mock import patch
 
@@ -15,5 +15,9 @@ from domonic.JSON import *
 
 # class TestCase(unittest.TestCase):
 
+#     def test_json(self):
+
+
+
 # if __name__ == '__main__':
 #     unittest.main()
